from functools import wraps
from . import requests

UNITS_METRIC = 'SI'
UNITS_IMPERIAL = 'SU'

CUSTOMS_PAYMENTS_CLIENT = 'DDU'
CUSTOMS_PAYMENTS_CUSTOMER = 'DAP'
CUSTOMS_PAYMENTS = 'DDP'

SERVICE_TYPE_EU = 'U'
SERVICE_TYPE_WORLD = 'P'
SERVICE_TYPE_WORLD_DOCUMENTS = 'D'
SERVICE_TYPE_DOMESTIC = 'N'

PAYMENT_SHIPPER = 'S'
PAYMENT_RECEIVER = 'R'
PAYMENT_BILLING = 'T'

DROP_OFF_REGULAR = 'REGULAR_PICKUP'
DROP_OFF_COURIER = 'REQUEST_COURIER'

CURRENCY_EUR = 'EUR'
CURRENCY_USD = 'USD'

CONTENT_DOCUMENTS = 'DOCUMENTS'
CONTENT_NON_DOCUMENTS = 'NON_DOCUMENTS'

EU_CODES = ['AT', 'BE', 'BG', 'DE', 'CY', 'CZ', 'DK', 'EE', 'ES', 'FI', 'FR', 'GB', 'GR', 'HR', 'HU', 'IE', 'IT', 'LT',
            'LU', 'LV', 'MC', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK']

WORLD_CODES = EU_CODES + ['AD', 'AE', 'AF', 'AG', 'AI', 'AL', 'AM', 'AO', 'AR', 'AT', 'AU', 'AW', 'AZ',
                          'BA', 'BB', 'BD', 'BE', 'BF', 'BH', 'BI', 'BJ', 'BM', 'BN', 'BO', 'BR', 'BS', 'BT', 'BW',
                          'BY', 'BZ', 'CA', 'CD', 'CF', 'CG', 'CH', 'CI', 'CK', 'CL', 'CM', 'CN', 'CO', 'CR', 'CU',
                          'DJ', 'DM', 'DO', 'DZ', 'EC', 'EG', 'ER', 'ET', 'FJ', 'FK', 'FO', 'GA', 'GD', 'GE', 'GG',
                          'GH', 'GI', 'GL', 'GM', 'GN', 'GP', 'GQ', 'GT', 'GW', 'GY', 'HK', 'HN', 'HT', 'ID', 'IL',
                          'IN', 'IQ', 'IR', 'IS', 'JE', 'JM', 'JO', 'JP', 'KG', 'KI', 'KM', 'KN', 'KP', 'KR', 'KV',
                          'KW', 'KY', 'KZ', 'LA', 'LB', 'LC', 'LI', 'LK', 'LR', 'LS', 'LV', 'LY', 'MA', 'MD', 'ME',
                          'MG', 'MK', 'ML', 'MM', 'MN', 'MO', 'MQ', 'MR', 'MS', 'MU', 'MV', 'MW', 'MX', 'MY', 'MZ',
                          'NA', 'NC', 'NE', 'NG', 'NI', 'NO', 'NP', 'NR', 'NU', 'NZ', 'OM', 'PA', 'PE', 'PF', 'PG',
                          'PH', 'PK', 'PY', 'QA', 'RE', 'RS', 'RU', 'RW', 'SA', 'SB', 'SC', 'SD', 'SG', 'SH', 'SL',
                          'SM', 'SN', 'SO', 'SR', 'SS', 'ST', 'SV', 'SY', 'SZ', 'TC', 'TD', 'TG', 'TH', 'TJ', 'TL',
                          'TN', 'TO', 'TR', 'TT', 'TV', 'TW', 'TZ', 'UA', 'UG', 'US', 'UY', 'UZ', 'VA', 'VC', 'VE',
                          'VG', 'VN', 'VU', 'WS', 'XB', 'XC', 'XE', 'XM', 'XN', 'XS', 'XY', 'YE', 'YT', 'ZA', 'ZM',
                          'ZW']

LABEL_DIR = 'labels/'

INVALID_CHARACTERS = (':', '&', '>', '<')
STR_MAX_LENGTH = 35


def validate_characters(func):
    @wraps(func)
    def wrapper_validate(f_name, *args, **kwargs):
        for value in list(args) + list(kwargs.values()):
            if value and any(char in str(value) for char in INVALID_CHARACTERS):
                raise AttributeError(f'ERROR: invalid characters ({INVALID_CHARACTERS}) in {value}')
        return func(f_name, *args, **kwargs)
    return wrapper_validate


def validate_country(country, codes=WORLD_CODES):
    if country not in codes:
        raise AttributeError(f'ERROR: country code in {country} is invalid, please enter a valid code')
    return country


def validate_length(input, length=STR_MAX_LENGTH):
    if input and len(input) > length:
        raise AttributeError(f'ERROR: max length is {length}, {input} was {len(input)}')
    return input


def validate_pickup(timestamp, pickup_around=None):
    if pickup_around and pickup_around < timestamp:
        raise ValueError(f'The Pickup Location Close Time ({pickup_around}) cannot be earlier than Pickup Time.')
    return timestamp, pickup_around


class _ContainerInterface:
    def __setitem__(self, key, value):
        self.__setattr__(key, value)

    def __getitem__(self, item):
        return self.__getattribute__(item)

    def __contains__(self, item):
        return hasattr(self, item)

    def get(self, item, default=None):
        if item in self and self[item]:
            return self[item]
        return default

    def __repr__(self):
        return f'<{self.__class__.__name__}: {self.__dict__}>'


class Package(_ContainerInterface):
    def __init__(self, width, length, height, weight, price=None, description=None, reference=None):
        self.width = width
        self.length = length
        self.height = height
        self.weight = weight
        self.price = price or 0.0
        self.description = description
        self.tracking_number = None
        self.reference = reference


class Address(_ContainerInterface):
    @validate_characters
    def __init__(self, street_lines, city, postal_code, country, **kwargs):
        self.street_lines = validate_length(street_lines)
        self.city = city
        self.postal_code = postal_code
        self.country = validate_country(country)
        self.street_lines2 = kwargs.get('street_lines2', 'N/A')
        self.street_lines3 = kwargs.get('street_lines3')


class Contact(_ContainerInterface):
    @validate_characters
    def __init__(self, name, phone, email=None, company=None):
        self.name = name
        self.phone = phone
        self.email = email or "null"
        self.company = company or name


class Person(Contact, Address):
    def __init__(self, **kwargs):
        Contact.__init__(self, kwargs.get('name'), kwargs.get('phone'), kwargs.get('email'), kwargs.get('company'))
        Address.__init__(self, **kwargs)


class Shipment(_ContainerInterface):
    def __init__(self, packages, sender, recipient, **kwargs):
        self.pickup_timestamp, self.pickup_around = validate_pickup(kwargs.get('pickup_timestamp'),
                                                                    kwargs.get('around'))
        self.description = validate_length(kwargs.get('description'))

        self.sender = sender
        self.recipient = recipient
        self.packages = packages
        self.content = kwargs.get('content', CONTENT_NON_DOCUMENTS)
        self.currency = kwargs.get('currency', CURRENCY_EUR)
        self.customs = kwargs.get('customs')
        self.declared_value = kwargs.get('value')
        self.deleted = False
        self.drop_off = kwargs.get('drop_off', DROP_OFF_REGULAR)
        self.id = None
        self.label = None
        self.payment = kwargs.get('payment', CUSTOMS_PAYMENTS)
        self.pickup_location = kwargs.get('location')
        self.pickup_instructions = kwargs.get('instructions')
        self.pickup_requested = False
        self.rate = None
        self.services = kwargs.get('services', [])
        self.shipping_payment = kwargs.get('shipping_payment', PAYMENT_SHIPPER)
        self.ship_timestamp = kwargs.get('ship_timestamp')
        self.type = kwargs.get('service', SERVICE_TYPE_WORLD)
        self.unit = kwargs.get('units', UNITS_METRIC)

        if self.sender.country == self.recipient.country:
            self.type = SERVICE_TYPE_DOMESTIC
        elif self.sender.country in EU_CODES and self.recipient.country in EU_CODES:
            self.type = SERVICE_TYPE_EU
        elif self.content != CONTENT_NON_DOCUMENTS:
            self.type = SERVICE_TYPE_WORLD_DOCUMENTS

        if self.type in [SERVICE_TYPE_DOMESTIC, SERVICE_TYPE_EU]:
            self.content = CONTENT_DOCUMENTS

    def set_customs(self, customs_values):
        self.customs = customs_values

    def set_pickup(self, timestamp, **kwargs):
        self.pickup_timestamp, self.pickup_around = validate_pickup(timestamp, kwargs.get('around'))
        self.pickup_location = kwargs.get('location')
        self.pickup_instructions = kwargs.get('instructions')

    def set_label(self, graphic_img, format='pdf'):
        from os import path, makedirs
        from base64 import decodebytes

        if not path.exists(LABEL_DIR):
            makedirs(LABEL_DIR)

        label_path = LABEL_DIR + str(self.id) + '.' + format.lower()
        with open(label_path, 'wb') as pdf:
            pdf.write(decodebytes(str.encode(graphic_img)))

        self.label = label_path
        return self.label

    def add_service(self, value, currency=CURRENCY_EUR):
        self.services.append({'value': value, 'currency': currency})



from requests import post as _post
from datetime import datetime, timedelta
from collections import OrderedDict as _dict


CONFIG = {
    'sandbox': True,
}


def set_credentials(credentials):
    CONFIG['account'] = credentials['account']
    CONFIG['username'] = credentials['username']
    CONFIG['password'] = credentials['password']


def set_sandbox(mode=True):
    CONFIG['sandbox'] = True if mode is True else False


def _get_endpoint(tail):
    return 'https://wsbexpress.dhl.com/rest/{}/{}'.format('sndpt' if CONFIG['sandbox'] else 'gbl', tail)


TRACKING_LAST = 'LAST_CHECK_POINT_ONLY'
TRACKING_ALL = 'ALL_CHECK_POINTS'

DATETIME_FORMAT_GMT = "%Y-%m-%dT%H:%M:%SGMT"
DATETIME_FORMAT = "%Y-%m-%d"
DATETIME_FORMAT_Z = "%Y-%m-%dT%H:%M:%SZ"
HOUR_MIN_FORMAT = "%H:%M"


def date_to_str(date, format=DATETIME_FORMAT_GMT):
    date_str = date.strftime(format)
    if format == DATETIME_FORMAT_GMT:
        if date.strftime("%z") == "":
            date_str += "+00:00"
        else:
            date_str += date.strftime("%z")[:3] + ':' + date.strftime("%z")[3:]

    return date_str


def post(endpoint, body):
    response = _post(endpoint, auth=(CONFIG['username'], CONFIG['password']), json=body)

    if not response.ok:
        response.code, response.error_type = response.status_code, response.reason
    else:
        # find 'Notification' dictionary
        response_body = response.json()
        for element in response_body.values():

            # special case for rate response
            if element.get('Provider'):
                element = element['Provider'][0]

            notification = element.get('Notification')
            if notification:
                code, message = notification[0]["@code"], notification[0]["Message"]
                if code != "0":
                    # there was a error, build new response with error data
                    response.code = code
                    response.error_type = message
                    response.status_code = 500

    return response


def _prepare_requested_packages(shipment):
    # prepare requested packages
    packages = []
    d_packages = {}
    for number, package in enumerate(shipment['packages']):
        str_number = str(number + 1)
        reference = package.get('reference', "Piece " + str_number)

        packages.append({
            '@number': str_number,
            'Weight': package['weight'],
            'Dimensions': {'Length': package['length'], 'Width': package['width'], 'Height': package['height']},
            'CustomerReferences': reference
        })

        d_packages[str_number] = package

    return packages, d_packages


def _ship(shipment, upper_case=False):
    def upper_values(dictionary):
        return {k: str(v).upper() for k, v in dictionary.items()}

    shipper, recipient = shipment['sender'], shipment['recipient']
    ship = _dict({
        'Shipper': {
            'Contact': {
                'PersonName': shipper['name'],
                'CompanyName': shipper['company'],
                'PhoneNumber': shipper['phone'],
                'EmailAddress': shipper['email']
            },
            'Address': {
                'StreetLines': shipper['street_lines'],
                'StreetLines2': shipper['street_lines2'],
                'City': shipper['city'],
                'PostalCode': shipper['postal_code'],
                'CountryCode': shipper['country']
            }
        },
        'Recipient': {
            'Contact': {
                'PersonName': recipient['name'],
                'CompanyName': recipient['company'],
                'PhoneNumber': recipient['phone'],
                'EmailAddress': recipient['email']
            },
            'Address': {
                'StreetLines': recipient['street_lines'],
                'StreetLines2': recipient['street_lines2'],
                'City': recipient['city'],
                'PostalCode': recipient['postal_code'],
                'CountryCode': recipient['country']
            }
        }
    })

    street_lines3 = shipper.get('street_lines3')
    if street_lines3:
        ship['Shipper']['Address']['StreetLines3'] = street_lines3

    street_lines3 = recipient.get('street_lines3')
    if street_lines3:
        ship['Recipient']['Address']['StreetLines3'] = street_lines3

    if upper_case:
        # Shipment request address fields must be upper case (A v1.3)
        ship['Shipper']['Address'] = upper_values(ship['Shipper']['Address'])
        ship['Recipient']['Address'] = upper_values(ship['Recipient']['Address'])

    return ship


def rate(shipment):
    # shipment date cannot be in the past or more than 10 days in future
    today = datetime.today()
    if shipment['ship_timestamp'] < today or shipment['ship_timestamp'] > today + timedelta(days=10):
        return {'error': 400,
                'message': f"shipment date cannot be in the past or more than 10 days in future [{shipment['ship_timestamp']}]"}

    packages = []
    for number, package in enumerate(shipment['packages']):
        packages.append({
            '@number': str(number + 1),
            'Weight': {'Value': package['weight']},
            'Dimensions': {'Length': package['length'], 'Width': package['width'], 'Height': package['height']}
        })

    if len(packages) is 1:
        packages = packages.pop()

    # prepare ship
    shipper, recipient = shipment['sender'], shipment['recipient']
    ship = _dict({
        'Shipper': {
            'City': shipper['city'],
            'PostalCode': shipper['postal_code'],
            'CountryCode': shipper['country']
        },
        'Recipient': {
            'City': recipient['city'],
            'PostalCode': recipient['postal_code'],
            'CountryCode': recipient['country']
        }
    })

    rate_request = _dict({
        'RateRequest': {
            'ClientDetails': None,
            'RequestedShipment': {
                'DropOffType': shipment['drop_off'],
                'ShipTimestamp': date_to_str(shipment['ship_timestamp']),
                'UnitOfMeasurement': shipment['unit'],
                'Content': shipment['content'],
                'DeclaredValue': shipment['declared_value'],
                'DeclaredValueCurrencyCode': shipment['currency'],
                'PaymentInfo': shipment['payment'],
                'NextBusinessDay': 'Y',
                'Account': CONFIG['account'],
                'Ship': ship,
                'Packages': {'RequestedPackages': packages},
            }
        }
    })

    # prepare special services if any
    services = []
    for s in shipment['services']:
        services.append({'Service': {
            'ServiceType': 'II',
            'ServiceValue': s['value'],
            'CurrencyCode': s['currency']
        }})

    if services:
        rate_request['RateRequest']['RequestedShipment'].update({'SpecialServices': services})

    response = post(_get_endpoint('RateRequest'), rate_request)
    if response.ok:
        rate_response = response.json()['RateResponse']['Provider'][0]
        # get best service rate
        best_service = rate_response['Service'][0]
        for service in rate_response['Service']:
            total_net = service['TotalNet']
            if total_net['Currency'] and 0 < total_net['Amount'] < best_service['TotalNet']['Amount']:
                best_service = service

        shipment['rate'] = best_service
        return rate_response

    return {'error': response.code, 'message': response.error_type}


def shipment(shipment):
    # prepare requested packages
    packages, d_packages = _prepare_requested_packages(shipment)

    shipment_request = _dict({
        'ShipmentRequest': {
            'RequestedShipment': {
                'ShipmentInfo': {
                    'DropOffType': shipment['drop_off'],
                    'ServiceType': shipment['type'],
                    'Account': CONFIG['account'],
                    'Currency': shipment['currency'],
                    'UnitOfMeasurement': shipment['unit']
                },
                'ShipTimestamp': date_to_str(shipment['ship_timestamp']),
                'PaymentInfo': shipment['payment'],
                'InternationalDetail': {
                    'Commodities': {
                        'Description': shipment['description']
                    },
                    'Content': shipment['content']
                },
                'Ship': _ship(shipment, upper_case=True),
                'Packages': {
                    'RequestedPackages': packages
                }
            }
        }
    })

    if shipment['customs']:
        shipment_request['ShipmentRequest']['RequestedShipment']['InternationalDetail']['Commodities'][
            'CustomsValue'] = shipment['customs']

    response = post(_get_endpoint('ShipmentRequest'), shipment_request)
    if response.ok:
        shipment_response = response.json()['ShipmentResponse']
        label = shipment_response['LabelImage'][0]
        shipment['id'] = shipment_response['ShipmentIdentificationNumber']
        _label, _label_format = label['GraphicImage'], label['LabelImageFormat']
        if hasattr(shipment, 'set_label'):
            shipment.set_label(_label, _label_format)
        else:
            shipment['label'] = (_label, _label_format)
        packages_response = shipment_response['PackagesResult']['PackageResult']
        # search for shipment packages in response in order to update them
        for r_package in packages_response:
            for s_package in packages:
                if r_package['@number'] == s_package['@number']:
                    # modify package
                    d_packages[s_package['@number']]['tracking_number'] = r_package['TrackingNumber']

        return shipment_response

    return {'error': response.code, 'message': response.error_type}


def tracking(waybill=None, level=TRACKING_ALL):
    from random import choice, randint
    from string import ascii_letters, digits

    message_reference = ''.join(choice(ascii_letters + digits) for _ in range(randint(28, 32)))
    if not waybill:
        # testing purposes
        waybill = choice([
            3898464710, 9296347550, 9296392420, 3567549160, 8423914660, 7195064870,
            7895183380, 1191166550, 7780184230, 5220325810, 7895111000, 7898385320,
            5138809620, 6424045110, 6768764450, 3314358460, 9545331390, 7895083910,
            8260591010, 4688745390, 7967566180])

    tracking_request = _dict({'trackShipmentRequest': {
        'trackingRequest': {
            'TrackingRequest': {
                'Request': {
                    'ServiceHeader': {
                        'MessageTime': date_to_str(datetime.now(), format=DATETIME_FORMAT_Z),
                        'MessageReference': message_reference
                    }
                },
                'AWBNumber': {'ArrayOfAWBNumberItem': waybill},
                'LevelOfDetails': level,
                'PiecesEnabled': 'B'
            }
        }
    }
    })

    response = post(_get_endpoint('TrackingRequest'), tracking_request)
    if response.ok:
        tracking_response = response.json()['trackShipmentRequestResponse']['trackingResponse']['TrackingResponse']
        return tracking_response

    return {'error': response.code, 'message': response.error_type}


def update(shipment):
    # prepare requested packages
    packages = []
    d_packages = {}
    for number, package in enumerate(shipment.packages):
        str_number = str(number + 1)
        packages.append({
            '@number': str_number,
            'Weight': package['weight'],
            'Dimensions': {'Length': package['length'], 'Width': packages['width'], 'Height': packages['height']},
        })

        d_packages[str_number] = package

    updated_shipment = _dict({
        'UpdateRequest': {
            'MessageId': "Sample_Message_Id",
            'UpdatedShipment': {
                'ShipmentInfo': {
                    'ServiceType': shipment['type'],
                    'Account': CONFIG['account'],
                    'ShipmentIdentificationNumber': shipment['id'],
                    'LabelType': 'PDF',
                    'LabelTemplate': 'ECOM_TC_A4',
                    'ArchiveLabelTemplate': 'ARCH_8x4',
                    'IncludePreviousPieceLabels': 'Y'
                },
                'OriginalShipDate': shipment.date_str(DATETIME_FORMAT),
                'Packages': {'RequestedPackages': packages}
            }
        }
    })

    response = post(_get_endpoint('UpdateRequest'), updated_shipment)
    if response.ok:
        update_response = response.json()['UpdateShipmentResponse']
        label = update_response['LabelImage'][0]
        shipment['id'] = update_response['ShipmentIdentificationNumber']
        shipment.set_label(label['GraphicImage'], label['LabelImageFormat'])
        packages_response = update_response['PackagesResult']['PackageResult']
        # search for shipment packages in response in order to update them
        for r_package in packages_response:
            for s_package in packages:
                if r_package['@number'] == s_package['@number']:
                    # modify package
                    d_packages[s_package['@number']].set_tracking_number(r_package['TrackingNumber'])

        return update_response

    return {'error': response.code, 'message': response.error_type}


def pickup(shipment):
    packages, d_packages = _prepare_requested_packages(shipment)

    pickup_request = _dict({
        "PickUpRequest": {
            "PickUpShipment": {
                "ShipmentInfo": {
                    "ServiceType": shipment['type'],
                    "Billing": {
                        "ShipperAccountNumber": CONFIG['account'],
                        "ShippingPaymentType": shipment['shipping_payment']
                    },
                    "UnitOfMeasurement": shipment['unit'],
                },
                "PickupTimestamp": date_to_str(shipment['pickup_timestamp']),
                "PickupLocationCloseTime": date_to_str(shipment['pickup_around'], HOUR_MIN_FORMAT),
                "SpecialPickupInstruction": shipment['pickup_instructions'],
                "PickupLocation": shipment['pickup_location'],
                "InternationalDetail": {
                    "Commodities": {
                        'Description': shipment['description']
                    }
                },
                "Ship": _ship(shipment),
                "Packages": {
                    "RequestedPackages": packages
                }
            }
        }
    })

    response = post(_get_endpoint('PickupRequest'), pickup_request)
    if response.ok:
        shipment['pickup_requested'] = True
        return response.json()['PickUpResponse']

    return {'error': response.code, 'message': response.error_type}



# -*- coding: utf-8 -*
import argparse
from pprint import pprint
from datetime import datetime
from . import Package, Shipment, Person, requests
from json import load, dumps


RATE_HELP = "The Rate request will return DHL’s product capabilities (products,\
 services and estimated delivery time) and prices (where applicable) for a \
 certain set of input data."
SHIPMENT_HELP = "The key elements in the response of the Shipment Request will \
be a base64 encoded PDF label and the Shipment and Piece identification numbers\
, which you can use for tracking on the DHL web site."
UPDATE_HELP = "The updateShipment request allows for additional pieces to be\
 added to a previously created shipment that has not been picked up by DHL\
  Express/had a scan against it."
TRACK_HELP = "The resulting response will provide tracking events at both the\
 Shipment and/or Piece events corresponding to the DHL Waybill(s) submitted."
PICKUP_HELP = "The requestPickup request allows users to request standalone\
 pickup requests for local and remote/import pickups."


def parse_file(file):
    with file as _file:
        conf_shipment = load(_file)

    credentials = {}
    for k in ['account', 'username', 'password']:
        credentials[k] = conf_shipment[k]
        del conf_shipment[k]
    requests.set_credentials(credentials)

    # create packages
    packages = []
    for p in conf_shipment['packages']:
        packages.append(Package(**p))

    sender = Person(**conf_shipment['sender'])
    recipient = Person(**conf_shipment['recipient'])

    # transform ship_timestamp to a date
    ship_timestamp = datetime.strptime(conf_shipment['ship_timestamp'], requests.DATETIME_FORMAT_GMT)

    # reuse conf_timestamp into Shipment object building
    del conf_shipment['packages']
    del conf_shipment['sender']
    del conf_shipment['recipient']
    del conf_shipment['ship_timestamp']

    return Shipment(
        sender=sender,
        recipient=recipient,
        packages=packages,
        ship_timestamp=ship_timestamp,
        **conf_shipment
    )


def main():
    parser = argparse.ArgumentParser(prog='python_dhl', description='DHL REST Webservice integration')
    parser.add_argument('-r', '--rate', help=RATE_HELP, type=argparse.FileType(encoding='UTF-8'))
    parser.add_argument('-s', '--shipment', help=SHIPMENT_HELP, type=argparse.FileType(encoding='UTF-8'))
    parser.add_argument('-u', '--update', help=UPDATE_HELP, type=argparse.FileType(encoding='UTF-8'))
    parser.add_argument('-t', '--track', help=TRACK_HELP, type=argparse.FileType(encoding='UTF-8'))
    parser.add_argument('-p', '--pickup', help=PICKUP_HELP, type=argparse.FileType(encoding='UTF-8'))
    parser.add_argument('-o', '--output', help="Output File", type=argparse.FileType('w', encoding='UTF-8'))
    parser.add_argument('-snd', '--sandbox', help="Set sandbox mode", default=False, nargs='?', type=bool)

    args = parser.parse_args()

    if args.sandbox:
        requests.set_sandbox()

    results = {}

    if args.rate:
        shipment = parse_file(args.rate)
        results = requests.rate(shipment)
    elif args.shipment:
        shipment = parse_file(args.shipment)
        results = requests.shipment(shipment)
    elif args.update:
        shipment = parse_file(args.update)
        results = requests.update(shipment)
    elif args.track:
        shipment = parse_file(args.track)
        results = requests.tracking(shipment)
    elif args.pickup:
        shipment = parse_file(args.pickup)
        results = requests.pickup(shipment)

    if results:
        if args.output:
            with args.output as output:
                output.write(dumps(results, indent=2, separators=(',', ': ')))
        else:
            pprint(results, width=100)



from setuptools import setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(
    name='pyDHL',
    version='0.3',
    packages=['pyDHL'],
    description='DHL REST Webservice integration',
    long_description=readme(),
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=[
        'requests',
    ],
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
    zip_safe=False,
    entry_points = {
      'console_scripts': ['pyDHL=pyDHL.command_line:main'],
    },
    author='SMACH Team',
    maintainer_email='pablo.riutort@smachteam.com',
)
